@@ -10,7 +10,13 @@ class _State(_Base):
     def __init__(self, token, host='https://api.dynamofl.com', metadata=None):
         super().__init__(token, host)
 
+        # Horizontal
         self.project_participants = []
+
+        # Vertical
+        self.label_participants = []
+        self.feature_participants = []
+
         self.datasources = {}
         self.instance_id = None
         self.metadata = metadata
@@ -47,10 +53,10 @@ class _State(_Base):
         if 'model_path' in self.datasources[datasource_key].trainers.get(trainer_key, {}):
             model_path = self.datasources[datasource_key].trainers[trainer_key]['model_path']
 
-        ext = project_info['modelType']
+        model_extension = project_info['modelType']
         current_round = project_info['currentRound']
         prev_round = self._get_last_fed_model_round(current_round, project_info['isComplete'])
-        federated_model_path = get_federated_path(project_key, model_path, ext, datasource_key, prev_round)
+        federated_model_path = get_federated_path(project_key, model_path, model_extension, datasource_key, prev_round)
 
         yes_stats = self._check_stats(project_info, datasource_key, prev_round)
         yes_submission = self._check_submissions(project_info, datasource_key, current_round)
@@ -74,7 +80,7 @@ class _State(_Base):
 
         # Train and push
         if not yes_submission:
-            new_model_path = get_trained_path(project_key, model_path, ext, datasource_key, current_round)
+            new_model_path = get_trained_path(project_key, model_path, model_extension, datasource_key, current_round)
 
             print(f'>>> ({project_key}-{datasource_key}) Training weights on local model...')
             train_res = train(datasource_key, federated_model_path, new_model_path, project_info, hyper_param_values)
@@ -104,7 +110,7 @@ class _State(_Base):
                         'datasource_key': i['datasourceKey'],
                         'trainer_key': i['trainerKey'],
                         'hyper_param_values': i['hyperParamValues'],
-                        'labeled': i['isLabelled']
+                        'labeled': i.get('isLabelled', True)
                     })
         if should_spawn_train:
             for p in self.project_participants:
@@ -126,14 +132,13 @@ class _State(_Base):
 
                 # todo
                 # take this out and put logic on the server
-                if info['isSemiSupervised']:
-                    if (info['currentRound'] == info['rounds']):
-                        continue
+                if (info['currentRound'] == info['rounds']):
+                    continue
 
                 self.train_and_test_callback(datasource_key, info)
 
     # creates a new datasource in the api
-    def attach_datasource(self, key, name=None, metadata=None):
+    def attach_datasource(self, key, name=None, metadata=None, type=None):
 
         while not self.instance_id:
             sleep(0.1)
@@ -145,6 +150,10 @@ class _State(_Base):
             params['metadata'] = self.metadata
         if metadata is not None:
             params['metadata'] = metadata
+        if type is not None and type != 'horizontal':
+            # Valid types are 'label' and 'feature'
+            params['type'] = type
+
 
         found_datasources = self._make_request('GET', '/datasources', params={'key': key}, list=True)
         if len(found_datasources):
@@ -153,7 +162,7 @@ class _State(_Base):
             self._make_request('POST', '/datasources', params=params)
 
 
-        ds = _Datasource(self, key)
+        ds = _Datasource(self, key, type)
         self.datasources[key] = ds
         self.initiate_project_participants(should_fetch_bridges=True, ds=key)
 
@@ -168,13 +177,16 @@ class _State(_Base):
     def get_user(self):
         return self._make_request('GET', '/user')
 
-    def create_project(self, base_model_path, params, dynamic_trainer_path=None):
+    def create_project(self, base_file, params, dynamic_trainer_path=None, type=None):
         j = self._make_request('POST', '/projects', params=params)
 
         project = _Project(self.token, self.host, j['key'])
-        project.push_model(base_model_path, None)
+        if type == 'horizontal':
+            project.push_model(base_file, None)
+        if type == 'vertical':
+            project.push_ids(base_file)
 
-        if dynamic_trainer_path:
+        if dynamic_trainer_path and type == 'horizontal':
             with open(dynamic_trainer_path, 'rb') as f:
                 self._make_request('POST', f'/projects/{project.key}/files', files={'file': f})
 
@@ -223,6 +235,30 @@ class _State(_Base):
         else:
             return len(user_stats)
 
+    def is_datasource_labeled(self, project_key=None, datasource_key=None):
+        """
+        Accepts a valid project_key and datasource_key.
+        Returns True if the datasource is labeled for the project; False otherwise
+        
+        """
+        if not datasource_key or not project_key:
+            raise Exception('project_key and datasource_key cannot be empty or None')
+        
+        try:
+            bridge = self._make_request('GET', '/bridges', params={
+                'projectKey': project_key,
+                'datasourceKey': datasource_key
+            })
+            
+            if len(bridge['data']) == 0:
+                raise Exception("datasource_key not associated with this project")
+            
+            return bridge['data'][0].get('isLabelled', True)
+
+        except Exception as e:
+            print("Something went wrong: {}".format(e))
+
+
 def get_federated_path(project_key, base, ext, ds, round):
     return f'{base}/federated_model_{project_key}_{ds}_{round}.{ext}'
 
